
%******************** Preparation part *************************************
close all;
clear all;
Fs=256000.0;        % Symbol rate

Nd = 200;           % Number of symbols that simulates in each loop

R=1;                % Number of oversamples

%************************* Filter initialization ***************************

N=64;                       % Number of taps
alpha=0.22;                 % Rolloff factor
[Tx_fir] = hrollfcoef(N,R,Fs,alpha,1);   %Transmitter filter coefficients
dyn_Tx=sum(abs(Tx_fir));
[Rx_fir] = hrollfcoef(N,R,Fs,alpha,0);   %Receiver filter coefficients 
dyn_Rx=sum(abs(Rx_fir));

%************************* Filter coefficient quantization ****************
b_h_tx=8;                 % number of bit for transmitter filter coefficients
b_tx=10;                  % number of bit for transmitter filter output 
b_CAN=12;                 % number of bit for receiver filter input  
b_h_rx=12;                % number of bit for receiver filter coefficients
b_rx=b_h_rx+b_CAN;    % number of bit for receiver filter output  

Tx_fir_q = round(Tx_fir.*2^(b_h_tx-1));
Rx_fir_q = round(Rx_fir.*2^(b_h_rx-1));

Tx_fir = Tx_fir_q.*2^(1-b_h_tx);
Rx_fir = Rx_fir_q.*2^(1-b_h_rx);


%**************************** START CALCULATION ****************************

nloop = 1;                                                       % simulation number of times
noe   = 0;
nod   = 0;


for ii=1:nloop
    
%****************************** Transmitter ********************************
    data = 2*(0.5 - rand(1,Nd));
    
    dataf = conv2(data,Rx_fir);   % filter 

    %******************************** PLOT ********************************
    figure;
    subplot(2,1,1); stairs(data); title('Input Data'); AXIS([1 Nd -1.1 1.1]);
    subplot(2,1,2); stairs(dataf); title('Filtered Data'); AXIS([1 Nd -1.1 1.1]);
    

end



%*************************** DATA FILES ***********************************

fid = fopen('DATA.dat','w');
fprintf(fid,'Input Data\n%d\n',Nd);
for ii=1:Nd
    fprintf(fid,'%d\n',data(1,ii));
end
fclose(fid);

fid = fopen('DATAF.dat','w');
fprintf(fid,'Filtered Data: dataf \n%d\n',length(dataf));
for ii=1:length(dataf)
    fprintf(fid,'%d \n',round(dataf(1,ii)*2^(b_rx-2)));
end
fclose(fid);

 
 % DATA for test of Rx filter : impulse response followed by step
 % response
 test_ich5 = zeros(1,length(data)); 
 test_ich5(1) = 0.5;
 test_ich5(40:length(data)) = 0.5;
 test_ich6 = conv2(test_ich5,Rx_fir); 
 fid = fopen('TEST_DATA.dat','w');
 fprintf(fid,'Data (for test): data\n%d\n',length(test_ich5));
 for ii=1:length(test_ich5)
    fprintf(fid,'%d \n',round(test_ich5(1,ii)*2^(b_CAN-1)));
 end
 fclose(fid);
 fid = fopen('TEST_DATAF.dat','w');
 fprintf(fid,'Filtered Data (for test): dataf\n%d\n',length(test_ich6));
 for ii=1:length(test_ich6)
    fprintf(fid,'%d  %d\n',round(test_ich6(1,ii)*2^(b_rx-2)));
 end
 fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fid = fopen('CDMA_pack.vhd','w');
fprintf(fid,'package CDMA_pack is\n');

% Input data
fprintf(fid,'constant input_data : array (0 to %d) of integer := (',Nd);
for ii=1:Nd-1
    fprintf(fid,'%d, ',data(1,ii));
end
fprintf(fid,'%d);\n\n',data(1,ii+1));

% Tx Fir coefficients
fprintf(fid,'constant H_Tx_fir : array (0 to %d) of integer := (',length(Tx_fir_q)-1);
for ii=1:length(Tx_fir_q)-1
    fprintf(fid,'%d, ',Tx_fir_q(ii));
end
fprintf(fid,'%d);\n\n',Tx_fir_q(ii+1));

% Rx Fir coefficients
fprintf(fid,'constant H_Rx_fir : array (0 to %d) of integer := (',length(Tx_fir_q)-1);
for ii=1:length(Rx_fir_q)-1
    fprintf(fid,'%d, ',Rx_fir_q(ii));
end
fprintf(fid,'%d);\n\n',Rx_fir_q(ii+1));



fprintf(fid,'end CDMA_pack;\n');
fclose(fid);

%******************************** end of file ********************************
